﻿/*
EonaCat.EventKit

Copyright 2025 EonaCat (Jeroen Saey)

	Apache License
    Version 2.0, January 2004
    http://www.apache.org/licenses/
    https://EonaCat.com/license/

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 
     OF SOFTWARE BY EONACAT (JEROEN SAEY)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License

*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;

namespace EonaCat.EventKit
{
    public static class EventSubscriptionManager
    {
        private static readonly Dictionary<object, Dictionary<string, Delegate>> _subscriptions = new();
        private static readonly HashSet<string> _activeEventStack = new();
        private static readonly Dictionary<string, int> _eventCount = new();

        public static void SubscribeWithCountLogging(
            object target,
            string eventName,
            Action<string, EventArgs?>? onEvent = null,
            SubscribeOptions? options = null)
        {
            Subscribe(target, eventName, onEvent, options);

            if (!_eventCount.ContainsKey(eventName))
            {
                _eventCount[eventName] = 0;
            }

            Action<object, EventArgs?> handler = (sender, args) =>
            {
                _eventCount[eventName]++;
                Console.WriteLine($"[EonaCat.EventKit] {eventName} fired {_eventCount[eventName]} times.");
                onEvent?.Invoke(eventName, args);
            };

            Subscribe(target, eventName, handler, options);
        }

        public static void SubscribeWithExecutionContext(
            object target,
            string eventName,
            Action<string, EventArgs?> onEvent,
            SynchronizationContext? syncContext = null,
            SubscribeOptions? options = null)
        {
            syncContext ??= SynchronizationContext.Current;

            Action<object, EventArgs?> handler = (sender, args) =>
            {
                if (syncContext != null)
                {
                    syncContext.Post(_ =>
                    {
                        onEvent(eventName, args);
                    }, null);
                }
                else
                {
                    onEvent(eventName, args);
                }
            };

            Subscribe(target, eventName, handler, options);
        }

        public static async void SubscribeWithExpiry(
            object target,
            string eventName,
            Action<string, EventArgs?> onEvent,
            TimeSpan expiryDuration,
            SubscribeOptions? options = null)
        {
            Subscribe(target, eventName, onEvent, options);

            await Task.Delay(expiryDuration);
            Unsubscribe(target, eventName);
        }

        public static void SubscribeWithRetry(
            object target,
            string eventName,
            Action<string, EventArgs?> onEvent,
            int maxRetries = 3,
            TimeSpan? retryInterval = null,
            SubscribeOptions? options = null)
        {
            retryInterval ??= TimeSpan.FromSeconds(1);
            var retries = 0;

            Action<object, EventArgs?> handler = (sender, args) =>
            {
                while (retries < maxRetries)
                {
                    try
                    {
                        onEvent(eventName, args);
                        break;
                    }
                    catch (Exception ex)
                    {
                        retries++;
                        if (retries == maxRetries)
                        {
                            Console.WriteLine($"[EonaCat.EventKit] {eventName} failed after {retries} retries. Error: {ex.Message}");
                        }
                        else
                        {
                            Console.WriteLine($"[EonaCat.EventKit] Retrying {eventName} due to error: {ex.Message}");
                            Task.Delay(retryInterval.Value).Wait();
                        }
                    }
                }
            };

            Subscribe(target, eventName, handler, options);
        }

        [AttributeUsage(AttributeTargets.Event)]
        public class EventFilterAttribute : Attribute
        {
            public string FilterName { get; }

            public EventFilterAttribute(string filterName)
            {
                FilterName = filterName;
            }
        }

        public static void SubscribeAllWithFilter(
            object target,
            string filterName,
            Action<string, EventArgs?> onEvent,
            SubscribeOptions? options = null)
        {
            foreach (var evt in target.GetType().GetEvents())
            {
                var attribute = evt.GetCustomAttribute<EventFilterAttribute>();
                if (attribute != null && attribute.FilterName == filterName)
                {
                    Subscribe(target, evt.Name, onEvent, options);
                }
            }
        }

        public static void SubscribeConcurrently(
    object target,
    string eventName,
    Func<string, EventArgs?, Task> onEventAsync,
    SubscribeOptions? options = null)
        {
            async void Handler(object sender, EventArgs args)
            {
                async Task HandleEventAsync()
                {
                    try
                    {
                        await onEventAsync(eventName, args);
                    }
                    catch (Exception ex)
                    {
                        if (options?.HandleExceptions ?? true)
                        {
                            Console.WriteLine($"[EonaCat.EventKit] Error handling event {eventName}: {ex.Message}");
                        }
                    }
                }
                await HandleEventAsync();
            }
            Subscribe(target, eventName, Handler, options);
        }


        public static int GetEventCount(string eventName)
        {
            return _eventCount.ContainsKey(eventName) ? _eventCount[eventName] : 0;
        }

        public static void SubscribeAll(
            object target,
            Func<EventInfo, bool>? filter = null,
            Action<string, EventArgs?>? onEvent = null,
            SubscribeOptions? options = null)
        {
            foreach (var evt in target.GetType().GetEvents())
            {
                if (filter == null || filter(evt))
                {
                    Subscribe(target, evt.Name, onEvent, options);
                }
            }
        }

        public static void Subscribe(
            object target,
            string eventName,
            Action<string, EventArgs?>? onEvent = null,
            SubscribeOptions? options = null)
        {
            if (target == null)
            {
                return;
            }

            options ??= new();

            var type = target.GetType();
            var eventInfo = type.GetEvent(eventName);
            if (eventInfo == null || !eventInfo.EventHandlerType.IsSubclassOf(typeof(Delegate)))
            {
                return;
            }

            if (_subscriptions.TryGetValue(target, out var dict) && dict.ContainsKey(eventName))
            {
                return;
            }

            Action<object, EventArgs?> handler = (s, e) =>
            {
                if (options.ReentrancySafe && _activeEventStack.Contains(eventName))
                {
                    return;
                }

                try
                {
                    _activeEventStack.Add(eventName);
                    onEvent?.Invoke(eventName, e);
                }
                catch (Exception ex)
                {
                    if (options.HandleExceptions)
                    {
                        Console.WriteLine($"[EonaCat.EventKit] Error in handler for {eventName}: {ex.Message}");
                    }
                }
                finally
                {
                    _activeEventStack.Remove(eventName);
                }
            };

            var del = CreateDelegate(eventInfo.EventHandlerType, handler);
            eventInfo.AddEventHandler(target, del);

            if (!_subscriptions.ContainsKey(target))
            {
                _subscriptions[target] = new();
            }

            _subscriptions[target][eventName] = del;
        }

        public static void SubscribeWeak(
            object target,
            string eventName,
            Action<string, EventArgs?>? onEvent = null,
            SubscribeOptions? options = null)
        {
            if (target == null)
            {
                return;
            }

            options ??= new();

            var type = target.GetType();
            var eventInfo = type.GetEvent(eventName);
            if (eventInfo == null || !eventInfo.EventHandlerType.IsSubclassOf(typeof(Delegate)))
            {
                return;
            }

            if (_subscriptions.TryGetValue(target, out var dict) && dict.ContainsKey(eventName))
            {
                return;
            }

            WeakEventHandler weakHandler = new WeakEventHandler(target, eventName, onEvent);

            var del = CreateDelegate(eventInfo.EventHandlerType, weakHandler.Handler);
            eventInfo.AddEventHandler(target, del);

            if (!_subscriptions.ContainsKey(target))
            {
                _subscriptions[target] = new();
            }

            _subscriptions[target][eventName] = del;
        }

        public static void Unsubscribe(object target, string eventName)
        {
            if (!_subscriptions.TryGetValue(target, out var dict) || !dict.ContainsKey(eventName))
            {
                return;
            }

            var eventInfo = target.GetType().GetEvent(eventName);
            eventInfo?.RemoveEventHandler(target, dict[eventName]);
            dict.Remove(eventName);

            if (dict.Count == 0)
            {
                _subscriptions.Remove(target);
            }
        }

        public static void UnsubscribeAll(object target)
        {
            if (!_subscriptions.TryGetValue(target, out var dict))
            {
                return;
            }

            foreach (var (eventName, handler) in dict)
            {
                target.GetType().GetEvent(eventName)?.RemoveEventHandler(target, handler);
            }

            _subscriptions.Remove(target);
        }

        public static IEnumerable<(string EventName, Delegate Handler)> ListSubscriptions(object target)
        {
            return _subscriptions.TryGetValue(target, out var dict)
                ? dict.Select(kv => (kv.Key, kv.Value))
                : Enumerable.Empty<(string, Delegate)>();
        }

        private static Delegate CreateDelegate(Type eventType, Action<object, EventArgs?> handler)
        {
            return Delegate.CreateDelegate(eventType, handler.Target!, handler.Method);
        }

        public static void SubscribeWithAutoUnsubscribeOnDispose(
            object target,
            string eventName,
            Action<string, EventArgs?>? onEvent = null,
            SubscribeOptions? options = null)
        {
            Subscribe(target, eventName, onEvent, options);

            if (target is IDisposable disposable)
            {
                var disposableWithEvent = new DisposableWithEvent(disposable);

                disposableWithEvent.Disposed += (sender, args) =>
                {
                    // Unsubscribe from the event when disposed
                    Unsubscribe(target, eventName);  
                };
            }
        }

        public class DisposableWithEvent : IDisposable
        {
            private readonly IDisposable _disposableTarget;

            // Event to be triggered when Dispose() is called
            public event EventHandler? Disposed;

            public DisposableWithEvent(IDisposable disposableTarget)
            {
                _disposableTarget = disposableTarget ?? throw new ArgumentNullException(nameof(disposableTarget));
            }

            public void Dispose()
            {
                OnDisposed();

                _disposableTarget.Dispose();
            }

            protected virtual void OnDisposed()
            {
                Disposed?.Invoke(this, EventArgs.Empty);
            }
        }

        public class WeakEventHandler
        {
            private WeakReference _targetReference;
            private string _eventName;
            private Action<string, EventArgs?> _onEvent;

            public WeakEventHandler(object target, string eventName, Action<string, EventArgs?> onEvent)
            {
                _targetReference = new WeakReference(target);
                _eventName = eventName;
                _onEvent = onEvent;
            }

            public void Handler(object? sender, EventArgs? args)
            {
                if (_targetReference.IsAlive)
                {
                    _onEvent(_eventName, args);
                }
                else
                {
                    Unsubscribe(sender, _eventName);
                }
            }
        }
    }
}
