﻿/*
EonaCat.EventKit

Copyright 2025 EonaCat (Jeroen Saey)

	Apache License
    Version 2.0, January 2004
    http://www.apache.org/licenses/
    https://EonaCat.com/license/

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 
     OF SOFTWARE BY EONACAT (JEROEN SAEY)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License

*/

using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.CSharp;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using Microsoft.CodeAnalysis.Text;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

[Generator]
public class EventSubscriptionSourceGenerator : ISourceGenerator
{
    private const string EventHandlerPrefix = "GeneratedEventHandler";

    public void Initialize(GeneratorInitializationContext context)
    {
        // Initialization logic (if any) can go here
    }

    public void Execute(GeneratorExecutionContext context)
    {
        var syntaxReceiver = context.SyntaxReceiver as SyntaxReceiver;
        if (syntaxReceiver == null)
        {
            return;
        }

        foreach (var classDeclaration in syntaxReceiver.Classes)
        {
            var className = classDeclaration.Identifier.Text;
            var eventSubscriptions = new StringBuilder();
            var unsubscribeMethods = new StringBuilder();
            var eventHandlers = new StringBuilder();

            foreach (var eventMember in classDeclaration.Members.OfType<EventDeclarationSyntax>())
            {
                var eventName = eventMember.Identifier.Text;
                var eventType = eventMember.Type.ToString();
                var eventHandlerMethod = GenerateEventHandlerMethod(eventMember);

                // Generate subscription code
                eventSubscriptions.AppendLine($@"
                    EonaCat.EventKit.EventSubscriptionManager.SubscribeWeak({className}.Instance, ""{eventName}"", {eventHandlerMethod});");

                // Generate unsubscription code
                unsubscribeMethods.AppendLine($@"
                    EonaCat.EventKit.EventSubscriptionManager.Unsubscribe({className}.Instance, ""{eventName}"");");

                // Add the generated event handler to the class
                eventHandlers.AppendLine(eventHandlerMethod);
            }

            var sourceCode = $@"
// <auto-generated/>
// This file is auto-generated by EonaCat.EventKit.
// Do not edit this file directly.

/*
EonaCat.EventKit

Copyright 2025 EonaCat (Jeroen Saey)

	Apache License
    Version 2.0, January 2004
    http://www.apache.org/licenses/
    https://EonaCat.com/license/

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 
     OF SOFTWARE BY EONACAT (JEROEN SAEY)

Licensed under the Apache License, Version 2.0 (the ""License"");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an ""AS IS"" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License

*/

namespace GeneratedEventSubscriptions
{{
    public static class {className}EventSubscriptions
    {{
        public static void Subscribe()
        {{
            {eventSubscriptions}
        }}

        public static void Unsubscribe()
        {{
            {unsubscribeMethods}
        }}

        // Event Handlers
        {eventHandlers}
    }}
}}";

            // Add the generated code as a new source file
            context.AddSource($"{className}EventSubscriptions.g.cs", SourceText.From(sourceCode, Encoding.UTF8));
        }
    }

    private string GenerateEventHandlerMethod(EventDeclarationSyntax eventDeclaration)
    {
        var eventType = eventDeclaration.Type.ToString();
        var eventHandlerMethodName = $"{EventHandlerPrefix}_{eventDeclaration.Identifier.Text}Handler";

        // Check if the event handler has parameters
        var eventHandlerParameters = eventType.Contains(",")
            ? eventType.Split(',').Select(param => param.Trim()).ToList()
            : new List<string> { eventType };

        var eventHandlerParametersText = string.Join(", ", eventHandlerParameters.Select(p => $"{p} args"));

        return $@"
        private static void {eventHandlerMethodName}(object sender, {string.Join(", ", eventHandlerParameters)})
        {{
            // Your event handler logic here
            // For example: log the event
            Console.WriteLine($""{eventDeclaration.Identifier.Text} event fired with parameters: {string.Join(", ", eventHandlerParameters)}"");
        }}";
    }
}

public class SyntaxReceiver : ISyntaxReceiver
{
    public List<ClassDeclarationSyntax> Classes { get; } = new List<ClassDeclarationSyntax>();

    public void OnVisitSyntaxNode(SyntaxNode syntaxNode)
    {
        if (syntaxNode is ClassDeclarationSyntax classDeclaration)
        {
            Classes.Add(classDeclaration);
        }
    }
}
